@@ -38,8 +38,8 @@ pub trait HdkT: Send + Sync {
     fn update(&self, update_input: UpdateInput) -> ExternResult<HeaderHash>;
     fn delete(&self, hash: HeaderHash) -> ExternResult<HeaderHash>;
     fn hash_entry(&self, entry: Entry) -> ExternResult<EntryHash>;
-    fn get(&self, get_input: GetInput) -> ExternResult<Option<Element>>;
-    fn get_details(&self, get_input: GetInput) -> ExternResult<Option<Details>>;
+    fn get(&self, get_input: Vec<GetInput>) -> ExternResult<Vec<Option<Element>>>;
+    fn get_details(&self, get_input: Vec<GetInput>) -> ExternResult<Vec<Option<Details>>>;
     fn must_get_entry(&self, must_get_entry_input: MustGetEntryInput) -> ExternResult<EntryHashed>;
     fn must_get_header(
         &self,
@@ -49,6 +49,11 @@ pub trait HdkT: Send + Sync {
         &self,
         must_get_valid_element_input: MustGetValidElementInput,
     ) -> ExternResult<Element>;
+    // CounterSigning
+    fn accept_countersigning_preflight_request(
+        &self,
+        preflight_request: PreflightRequest,
+    ) -> ExternResult<PreflightRequestAcceptance>;
     // Info
     fn agent_info(&self, agent_info_input: ()) -> ExternResult<AgentInfo>;
     fn app_info(&self, app_info_input: ()) -> ExternResult<AppInfo>;
@@ -58,17 +63,20 @@ pub trait HdkT: Send + Sync {
     // Link
     fn create_link(&self, create_link_input: CreateLinkInput) -> ExternResult<HeaderHash>;
     fn delete_link(&self, add_link_header: HeaderHash) -> ExternResult<HeaderHash>;
-    fn get_links(&self, get_links_input: GetLinksInput) -> ExternResult<Links>;
-    fn get_link_details(&self, get_links_input: GetLinksInput) -> ExternResult<LinkDetails>;
+    fn get_links(&self, get_links_input: Vec<GetLinksInput>) -> ExternResult<Vec<Links>>;
+    fn get_link_details(
+        &self,
+        get_links_input: Vec<GetLinksInput>,
+    ) -> ExternResult<Vec<LinkDetails>>;
     // P2P
-    fn call(&self, call: Call) -> ExternResult<ZomeCallResponse>;
-    fn call_remote(&self, call_remote: CallRemote) -> ExternResult<ZomeCallResponse>;
+    fn call(&self, call: Vec<Call>) -> ExternResult<Vec<ZomeCallResponse>>;
+    fn call_remote(&self, call_remote: Vec<CallRemote>) -> ExternResult<Vec<ZomeCallResponse>>;
     fn emit_signal(&self, app_signal: AppSignal) -> ExternResult<()>;
     fn remote_signal(&self, remote_signal: RemoteSignal) -> ExternResult<()>;
     // Random
     fn random_bytes(&self, number_of_bytes: u32) -> ExternResult<Bytes>;
     // Time
-    fn sys_time(&self, sys_time_input: ()) -> ExternResult<core::time::Duration>;
+    fn sys_time(&self, sys_time_input: ()) -> ExternResult<Timestamp>;
     fn schedule(&self, execute_after: std::time::Duration) -> ExternResult<()>;
     fn sleep(&self, wake_after: std::time::Duration) -> ExternResult<()>;
     // Trace
@@ -132,10 +140,10 @@ impl HdkT for ErrHdk {
     fn hash_entry(&self, _: Entry) -> ExternResult<EntryHash> {
         Self::err()
     }
-    fn get(&self, _: GetInput) -> ExternResult<Option<Element>> {
+    fn get(&self, _: Vec<GetInput>) -> ExternResult<Vec<Option<Element>>> {
         Self::err()
     }
-    fn get_details(&self, _: GetInput) -> ExternResult<Option<Details>> {
+    fn get_details(&self, _: Vec<GetInput>) -> ExternResult<Vec<Option<Details>>> {
         Self::err()
     }
     fn must_get_entry(&self, _: MustGetEntryInput) -> ExternResult<EntryHashed> {
@@ -147,6 +155,13 @@ impl HdkT for ErrHdk {
     fn must_get_valid_element(&self, _: MustGetValidElementInput) -> ExternResult<Element> {
         Self::err()
     }
+    // CounterSigning
+    fn accept_countersigning_preflight_request(
+        &self,
+        _: PreflightRequest,
+    ) -> ExternResult<PreflightRequestAcceptance> {
+        Self::err()
+    }
     fn agent_info(&self, _: ()) -> ExternResult<AgentInfo> {
         Self::err()
     }
@@ -169,17 +184,17 @@ impl HdkT for ErrHdk {
     fn delete_link(&self, _: HeaderHash) -> ExternResult<HeaderHash> {
         Self::err()
     }
-    fn get_links(&self, _: GetLinksInput) -> ExternResult<Links> {
+    fn get_links(&self, _: Vec<GetLinksInput>) -> ExternResult<Vec<Links>> {
         Self::err()
     }
-    fn get_link_details(&self, _: GetLinksInput) -> ExternResult<LinkDetails> {
+    fn get_link_details(&self, _: Vec<GetLinksInput>) -> ExternResult<Vec<LinkDetails>> {
         Self::err()
     }
     // P2P
-    fn call(&self, _: Call) -> ExternResult<ZomeCallResponse> {
+    fn call(&self, _: Vec<Call>) -> ExternResult<Vec<ZomeCallResponse>> {
         Self::err()
     }
-    fn call_remote(&self, _: CallRemote) -> ExternResult<ZomeCallResponse> {
+    fn call_remote(&self, _: Vec<CallRemote>) -> ExternResult<Vec<ZomeCallResponse>> {
         Self::err()
     }
     fn emit_signal(&self, _: AppSignal) -> ExternResult<()> {
@@ -193,7 +208,7 @@ impl HdkT for ErrHdk {
         Self::err()
     }
     // Time
-    fn sys_time(&self, _: ()) -> ExternResult<core::time::Duration> {
+    fn sys_time(&self, _: ()) -> ExternResult<Timestamp> {
         Self::err()
     }
     fn schedule(&self, _: std::time::Duration) -> ExternResult<()> {
@@ -257,7 +272,6 @@ impl HdkT for HostHdk {
     fn query(&self, filter: ChainQueryFilter) -> ExternResult<Vec<Element>> {
         host_call::<ChainQueryFilter, Vec<Element>>(__query, filter)
     }
-
     fn sign(&self, sign: Sign) -> ExternResult<Signature> {
         host_call::<Sign, Signature>(__sign, sign)
     }
@@ -267,7 +281,6 @@ impl HdkT for HostHdk {
     fn verify_signature(&self, verify_signature: VerifySignature) -> ExternResult<bool> {
         host_call::<VerifySignature, bool>(__verify_signature, verify_signature)
     }
-
     fn create(&self, entry_with_def_id: EntryWithDefId) -> ExternResult<HeaderHash> {
         host_call::<EntryWithDefId, HeaderHash>(__create, entry_with_def_id)
     }
@@ -280,11 +293,11 @@ impl HdkT for HostHdk {
     fn hash_entry(&self, entry: Entry) -> ExternResult<EntryHash> {
         host_call::<Entry, EntryHash>(__hash_entry, entry)
     }
-    fn get(&self, get_input: GetInput) -> ExternResult<Option<Element>> {
-        host_call::<GetInput, Option<Element>>(__get, get_input)
+    fn get(&self, get_inputs: Vec<GetInput>) -> ExternResult<Vec<Option<Element>>> {
+        host_call::<Vec<GetInput>, Vec<Option<Element>>>(__get, get_inputs)
     }
-    fn get_details(&self, get_input: GetInput) -> ExternResult<Option<Details>> {
-        host_call::<GetInput, Option<Details>>(__get_details, get_input)
+    fn get_details(&self, get_inputs: Vec<GetInput>) -> ExternResult<Vec<Option<Details>>> {
+        host_call::<Vec<GetInput>, Vec<Option<Details>>>(__get_details, get_inputs)
     }
     fn must_get_entry(&self, must_get_entry_input: MustGetEntryInput) -> ExternResult<EntryHashed> {
         host_call::<MustGetEntryInput, EntryHashed>(__must_get_entry, must_get_entry_input)
@@ -307,6 +320,16 @@ impl HdkT for HostHdk {
             must_get_valid_element_input,
         )
     }
+    // CounterSigning
+    fn accept_countersigning_preflight_request(
+        &self,
+        preflight_request: PreflightRequest,
+    ) -> ExternResult<PreflightRequestAcceptance> {
+        host_call::<PreflightRequest, PreflightRequestAcceptance>(
+            __accept_countersigning_preflight_request,
+            preflight_request,
+        )
+    }
     fn agent_info(&self, _: ()) -> ExternResult<AgentInfo> {
         host_call::<(), AgentInfo>(__agent_info, ())
     }
@@ -322,25 +345,26 @@ impl HdkT for HostHdk {
     fn call_info(&self, _: ()) -> ExternResult<CallInfo> {
         host_call::<(), CallInfo>(__call_info, ())
     }
-
     fn create_link(&self, create_link_input: CreateLinkInput) -> ExternResult<HeaderHash> {
         host_call::<CreateLinkInput, HeaderHash>(__create_link, create_link_input)
     }
     fn delete_link(&self, add_link_header: HeaderHash) -> ExternResult<HeaderHash> {
         host_call::<HeaderHash, HeaderHash>(__delete_link, add_link_header)
     }
-    fn get_links(&self, get_links_input: GetLinksInput) -> ExternResult<Links> {
-        host_call::<GetLinksInput, Links>(__get_links, get_links_input)
+    fn get_links(&self, get_links_input: Vec<GetLinksInput>) -> ExternResult<Vec<Links>> {
+        host_call::<Vec<GetLinksInput>, Vec<Links>>(__get_links, get_links_input)
     }
-    fn get_link_details(&self, get_links_input: GetLinksInput) -> ExternResult<LinkDetails> {
-        host_call::<GetLinksInput, LinkDetails>(__get_link_details, get_links_input)
+    fn get_link_details(
+        &self,
+        get_links_input: Vec<GetLinksInput>,
+    ) -> ExternResult<Vec<LinkDetails>> {
+        host_call::<Vec<GetLinksInput>, Vec<LinkDetails>>(__get_link_details, get_links_input)
     }
-
-    fn call(&self, call: Call) -> ExternResult<ZomeCallResponse> {
-        host_call::<Call, ZomeCallResponse>(__call, call)
+    fn call(&self, call: Vec<Call>) -> ExternResult<Vec<ZomeCallResponse>> {
+        host_call::<Vec<Call>, Vec<ZomeCallResponse>>(__call, call)
     }
-    fn call_remote(&self, call_remote: CallRemote) -> ExternResult<ZomeCallResponse> {
-        host_call::<CallRemote, ZomeCallResponse>(__call_remote, call_remote)
+    fn call_remote(&self, call_remote: Vec<CallRemote>) -> ExternResult<Vec<ZomeCallResponse>> {
+        host_call::<Vec<CallRemote>, Vec<ZomeCallResponse>>(__call_remote, call_remote)
     }
     fn emit_signal(&self, app_signal: AppSignal) -> ExternResult<()> {
         host_call::<AppSignal, ()>(__emit_signal, app_signal)
@@ -348,13 +372,11 @@ impl HdkT for HostHdk {
     fn remote_signal(&self, remote_signal: RemoteSignal) -> ExternResult<()> {
         host_call::<RemoteSignal, ()>(__remote_signal, remote_signal)
     }
-
     fn random_bytes(&self, number_of_bytes: u32) -> ExternResult<Bytes> {
         host_call::<u32, Bytes>(__random_bytes, number_of_bytes)
     }
-
-    fn sys_time(&self, _: ()) -> ExternResult<core::time::Duration> {
-        host_call::<(), core::time::Duration>(__sys_time, ())
+    fn sys_time(&self, _: ()) -> ExternResult<Timestamp> {
+        host_call::<(), Timestamp>(__sys_time, ())
     }
     fn schedule(&self, execute_after: std::time::Duration) -> ExternResult<()> {
         host_call::<std::time::Duration, ()>(__schedule, execute_after)
@@ -362,11 +384,9 @@ impl HdkT for HostHdk {
     fn sleep(&self, wake_after: std::time::Duration) -> ExternResult<()> {
         host_call::<std::time::Duration, ()>(__sleep, wake_after)
     }
-
     fn trace(&self, trace_msg: TraceMsg) -> ExternResult<()> {
         host_call::<TraceMsg, ()>(__trace, trace_msg)
     }
-
     fn create_x25519_keypair(&self, _: ()) -> ExternResult<X25519PubKey> {
         host_call::<(), X25519PubKey>(__create_x25519_keypair, ())
     }
